@@ -1,4 +1,4 @@
-from typing import Any, TYPE_CHECKING, List
+from typing import Any, Optional, TYPE_CHECKING, List
 
 from archinstall.default_profiles.profile import Profile, ProfileType
 
@@ -20,6 +20,10 @@ class XorgProfile(Profile):
 			support_gfx_driver=True
 		)
 
+	def preview_text(self) -> Optional[str]:
+		text = str(_('Environment type: {}')).format(self.profile_type.value)
+		return text + '\n' + self.packages_text()
+
 	@property
 	def packages(self) -> List[str]:
 		return [
