@@ -83,7 +83,9 @@ class DeviceHandler(object):
 					_PartitionInfo.from_partition(
 						partition,
 						fs_type,
+						lsblk_info.partn,
 						lsblk_info.partuuid,
+						lsblk_info.uuid,
 						lsblk_info.mountpoints,
 						subvol_infos
 					)
@@ -131,6 +133,20 @@ class DeviceHandler(object):
 		lsblk = get_lsblk_info(dev_path)
 		return Path(f'/dev/{lsblk.pkname}')
 
+	def get_unique_path_for_device(self, dev_path: Path) -> Optional[Path]:
+		paths = Path('/dev/disk/by-id').glob('*')
+		linked_targets = {p.resolve(): p for p in paths}
+		linked_wwn_targets = {p: linked_targets[p] for p in linked_targets
+			if p.name.startswith('wwn-') or p.name.startswith('nvme-eui.')}
+
+		if dev_path in linked_wwn_targets:
+			return linked_wwn_targets[dev_path]
+
+		if dev_path in linked_targets:
+			return linked_targets[dev_path]
+
+		return None
+
 	def get_uuid_for_path(self, path: Path) -> Optional[str]:
 		partition = self.find_partition(path)
 		return partition.partuuid if partition else None
@@ -152,20 +168,19 @@ class DeviceHandler(object):
 			mountpoint = Path(common_prefix)
 
 		try:
-			result = SysCommand(f'btrfs subvolume list {mountpoint}')
+			result = SysCommand(f'btrfs subvolume list {mountpoint}').decode()
 		except SysCallError as err:
 			debug(f'Failed to read btrfs subvolume information: {err}')
 			return subvol_infos
 
 		try:
-			if decoded := result.decode('utf-8'):
-				# ID 256 gen 16 top level 5 path @
-				for line in decoded.splitlines():
-					# expected output format:
-					# ID 257 gen 8 top level 5 path @home
-					name = Path(line.split(' ')[-1])
-					sub_vol_mountpoint = lsblk_info.btrfs_subvol_info.get(name, None)
-					subvol_infos.append(_BtrfsSubvolumeInfo(name, sub_vol_mountpoint))
+			# ID 256 gen 16 top level 5 path @
+			for line in result.splitlines():
+				# expected output format:
+				# ID 257 gen 8 top level 5 path @home
+				name = Path(line.split(' ')[-1])
+				sub_vol_mountpoint = lsblk_info.btrfs_subvol_info.get(name, None)
+				subvol_infos.append(_BtrfsSubvolumeInfo(name, sub_vol_mountpoint))
 		except json.decoder.JSONDecodeError as err:
 			error(f"Could not decode lsblk JSON: {result}")
 			raise err
@@ -377,7 +392,6 @@ class DeviceHandler(object):
 		attempts = 3
 		lsblk_info: Optional[LsblkInfo] = None
 
-		self.partprobe(path)
 		for attempt_nr in range(attempts):
 			time.sleep(attempt_nr + 1)
 			lsblk_info = get_lsblk_info(path)
@@ -385,8 +399,6 @@ class DeviceHandler(object):
 			if lsblk_info.partn and lsblk_info.partuuid and lsblk_info.uuid:
 				break
 
-			self.partprobe(path)
-
 		if not lsblk_info:
 			debug(f'Unable to get partition information: {path}')
 			raise DiskError(f'Unable to get partition information: {path}')
@@ -525,8 +537,6 @@ class DeviceHandler(object):
 			requires_delete = modification.wipe is False
 			self._perform_partitioning(part_mod, modification.device, disk, requires_delete=requires_delete)
 
-		self.partprobe(modification.device.device_info.path)
-
 	def mount(
 		self,
 		dev_path: Path,
@@ -598,7 +608,12 @@ class DeviceHandler(object):
 						path = Path(part_info.disk.device.path)
 						part_mods.setdefault(path, [])
 						part_mod = PartitionModification.from_existing_partition(part_info)
-						part_mod.mountpoint = mountpoint.root / mountpoint.relative_to(base_mountpoint)
+						if part_mod.mountpoint:
+							part_mod.mountpoint = mountpoint.root / mountpoint.relative_to(base_mountpoint)
+						else:
+							for subvol in part_mod.btrfs_subvols:
+								if sm := subvol.mountpoint:
+									subvol.mountpoint = sm.root / sm.relative_to(base_mountpoint)
 						part_mods[path].append(part_mod)
 						break
 
