@@ -90,14 +90,14 @@ class Luks2:
 		# Retry formatting the volume because archinstall can some times be too quick
 		# which generates a "Device /dev/sdX does not exist or access denied." between
 		# setting up partitions and us trying to encrypt it.
-		for retry_attempt in range(storage['DISK_RETRY_ATTEMPTS']):
+		for retry_attempt in range(storage['DISK_RETRY_ATTEMPTS'] + 1):
 			try:
 				SysCommand(cryptsetup_args)
 				break
 			except SysCallError as err:
 				time.sleep(storage['DISK_TIMEOUTS'])
 
-				if retry_attempt != storage['DISK_RETRY_ATTEMPTS'] - 1:
+				if retry_attempt != storage['DISK_RETRY_ATTEMPTS']:
 					continue
 
 				if err.exit_code == 1:
@@ -116,7 +116,7 @@ class Luks2:
 		command = f'/usr/bin/cryptsetup luksUUID {self.luks_dev_path}'
 
 		try:
-			return SysCommand(command).decode().strip()  # type: ignore
+			return SysCommand(command).decode()
 		except SysCallError as err:
 			info(f'Unable to get UUID for Luks device: {self.luks_dev_path}')
 			raise err
@@ -162,7 +162,6 @@ class Luks2:
 
 		# Get crypt-information about the device by doing a reverse lookup starting with the partition path
 		# For instance: /dev/sda
-		disk.device_handler.partprobe(self.luks_dev_path)
 		lsblk_info = disk.get_lsblk_info(self.luks_dev_path)
 
 		# For each child (sub-partition/sub-device)
