@@ -1,4 +1,3 @@
-import os
 from pathlib import Path
 from typing import Any, TYPE_CHECKING, Optional
 
@@ -15,7 +14,6 @@ from archinstall.lib.mirrors import use_mirrors, add_custom_mirrors
 from archinstall.lib.models import AudioConfiguration
 from archinstall.lib.models.bootloader import Bootloader
 from archinstall.lib.models.network_configuration import NetworkConfiguration
-from archinstall.lib.networking import check_mirror_reachable
 from archinstall.lib.profile.profiles_handler import profile_handler
 
 if TYPE_CHECKING:
@@ -56,6 +54,8 @@ def ask_user_questions():
 	# Ask which boot-loader to use (will only ask if we're in UEFI mode, otherwise will default to GRUB)
 	global_menu.enable('bootloader')
 
+	global_menu.enable('uki')
+
 	global_menu.enable('swap')
 
 	# Get the hostname for the machine
@@ -111,6 +111,7 @@ def perform_installation(mountpoint: Path):
 	# Retrieve list of additional repositories and set boolean values appropriately
 	enable_testing = 'testing' in archinstall.arguments.get('additional-repositories', [])
 	enable_multilib = 'multilib' in archinstall.arguments.get('additional-repositories', [])
+	run_mkinitcpio = not archinstall.arguments.get('uki')
 	locale_config: locale.LocaleConfiguration = archinstall.arguments['locale_config']
 	disk_encryption: disk.DiskEncryption = archinstall.arguments.get('disk_encryption', None)
 
@@ -141,6 +142,7 @@ def perform_installation(mountpoint: Path):
 		installation.minimal_installation(
 			testing=enable_testing,
 			multilib=enable_multilib,
+			mkinitcpio=run_mkinitcpio,
 			hostname=archinstall.arguments.get('hostname', 'archlinux'),
 			locale_config=locale_config
 		)
@@ -154,7 +156,10 @@ def perform_installation(mountpoint: Path):
 		if archinstall.arguments.get("bootloader") == Bootloader.Grub and SysInfo.has_uefi():
 			installation.add_additional_packages("grub")
 
-		installation.add_bootloader(archinstall.arguments["bootloader"])
+		installation.add_bootloader(
+			archinstall.arguments["bootloader"],
+			archinstall.arguments["uki"]
+		)
 
 		# If user selected to copy the current ISO network configuration
 		# Perform a copy of the config
@@ -225,11 +230,6 @@ def perform_installation(mountpoint: Path):
 	debug(f"Disk states after installing: {disk.disk_layouts()}")
 
 
-if archinstall.arguments.get('skip-mirror-check', False) is False and check_mirror_reachable() is False:
-	log_file = os.path.join(archinstall.storage.get('LOG_PATH', None), archinstall.storage.get('LOG_FILE', None))
-	info(f"Arch Linux mirrors are not reachable. Please check your internet connection and the log file '{log_file}'.")
-	exit(1)
-
 if not archinstall.arguments.get('silent'):
 	ask_user_questions()
 
